/*
 * SMART_PLANT_POT.c
 *
 * Created: 18-May-18 23:21:57
 * Author : user
 */ 

#define F_CPU 16000000
#include <avr/io.h>
#include <util/delay.h>

#define LED_RED    PORTD0
#define LED_YELLOW PORTD1
#define LED_BLUE   PORTD2
#define POMP       PORTD3
#define SENSOR_TEMPERATURE   PORTB0
#define SENSOR_LIGHT         PORTB1
#define SENSOR_HUMIDITY_AIR  PORTB2
#define SENSOR_HUMIDITY_SOIL PORTB3

void init() {
	// init LEDs
	DDRD |= (1 << LED_RED);
	DDRD |= (1 << LED_YELLOW);
	DDRD |= (1 << LED_BLUE);

	// init pomp
	DDRD |= (1 << POMP);

	// init temperature sensor
	DDRB &= ~(1 << SENSOR_TEMPERATURE);
	PORTB |= (1 << SENSOR_TEMPERATURE);

	// init light sensor
	DDRB &= ~(1 << SENSOR_LIGHT);
	PORTB |= (1 << SENSOR_LIGHT);

	// init air humidity sensor
	DDRB &= ~(1 << SENSOR_HUMIDITY_AIR);
	PORTB |= (1 << SENSOR_HUMIDITY_AIR);

	// init soil humidity sensor
	DDRB &= ~(1 << SENSOR_HUMIDITY_SOIL);
	PORTB |= (1 << SENSOR_HUMIDITY_SOIL);
}

void turn_on_led_red() {
	PORTD |= (1 << LED_RED);
}

void turn_on_led_yellow() {
	PORTD |= (1 << LED_YELLOW);
}

void turn_on_led_blue() {
	PORTD |= (1 << LED_BLUE);
}

void turn_on_pomp() {
	PORTD |= (1 << POMP);
}

void turn_off_led_red() {
	PORTD &= ~(1 << LED_RED);
}

void turn_off_led_yellow() {
	PORTD &= ~(1 << LED_YELLOW);
}

void turn_off_led_blue() {
	PORTD &= ~(1 << LED_BLUE);
}

void turn_off_pomp() {
	PORTD &= ~(1 << POMP);
}

void blink_led_blue() {
	PORTD ^= (1 << LED_BLUE);
	_delay_ms(200);
}

int is_temperature_hight() {
	return (PINB & (1 << SENSOR_TEMPERATURE)) ? 0 : 1;
}

int is_enough_light() {
	return (PINB & (1 << SENSOR_LIGHT)) ? 0 : 1;
}

int is_air_humidity_hight() {
	return (PINB & (1 << SENSOR_HUMIDITY_AIR)) ? 0 : 1;
}

int is_soild_wet() {
	return (PINB & (1 << SENSOR_HUMIDITY_SOIL)) ? 0 : 1;
}

int main(void)
{
	init();

	while (1) {
		// control yellow LED
		is_enough_light() ?	turn_off_led_yellow() : turn_on_led_yellow();

		// control red LED
		is_temperature_hight() ? turn_on_led_red() : turn_off_led_red();

		// control blue LED and the water pomp
		if (is_soild_wet()) {
			turn_off_pomp();
			turn_off_led_blue();
		} else {
			if (is_air_humidity_hight()) {
				turn_off_pomp();
				blink_led_blue();
			} else {
				turn_on_pomp();
				turn_on_led_blue();
			}
		}
	}
	 
	return 0;
}

